; #############################################################################
; #############################################################################
; #############################################################################
TWI_INI:
	; SCL(Hz) = CPU(Hz) / (16 + 2 * TWBR * 4^TWPS)	
	; Beispiel:										
	; 100.000 = 16.000.000 / (16 + 2 * 18 * 4^1)	

	; TWI TWBR 				Bit RATE Register	
	ldi temp, TWI_BIT_RATE
	STS TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, TWI_PRESCALER
	STS TWSR, temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	clr temp
	STS TWAR,temp
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|0<<TWIE
	STS TWCR,temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
TWI_AUS:
	ldi temp, 1<<TWINT|0<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|0<<TWEN|0<<TWIE
	STS TWCR,temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																			   
; START-ROUTINE	(MASTER TRANSMITTER)										   
;																			   
; INPUT:	temp1 <= ADRESSE SLAVE											   
; rcall TWI_MASTER_TRANSMITTER_START	; INP: temp1(SLA) out: FEHLER_CODE		
TWI_MASTER_TRANSMITTER_START_TEMP1:
	WDR
	; 	FEHLER lschen	
	clr FEHLER_CODE
	; START								
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)
	STS TWCR, temp	
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	LDS temp,TWSR													
	andi temp, 0xF8
	cpi temp,  0x08							; 0x08 START erfolgreich
	breq TWI_MASTER_TRANSMITTER_START_w
	cpi temp,  0x10							; 0x10 (repeated START)
	breq TWI_MASTER_TRANSMITTER_START_w
	; ERROR		
	ldi temp,(ERROR_TWI_MT_START)
	mov FEHLER_CODE, temp
	ret
TWI_MASTER_TRANSMITTER_START_w:
	; Load SLA + "W"  (W=0) 			
	LSL  temp1								; 1 x links
	STS TWDR, temp1
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	STS TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	LDS temp,TWSR												
	andi temp, 0xF8
	cpi temp,  0x18							; 0x18
	brne TWI_MASTER_TRANSMITTER_START_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_ERROR:
	; ERROR		
	ldi temp,(ERROR_TWI_MT_SLA)
	mov FEHLER_CODE, temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; Ein Zeichen zum SLAVE zu senden										   	   
; 																			   
; INPUT:	temp1 <= DATEN													   
; 																			   
; 	rcall TWI_MASTER_TRANSMITTER_DATEN	; inp: temp1 OUT: FEHLER_CODE		   
TWI_MASTER_TRANSMITTER_DATA_TEMP1:
	; FEHLERPRFUNG						
	tst FEHLER_CODE							; Fehler =0 ?	
	breq TWI_MASTER_TRANSMITTER_DATA_w		; ja			
	ret										; zurck		
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_DATA_w:
	; 	Load DATA						
	STS TWDR, temp1							
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	STS TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	LDS temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x28	; 0x28
	brne TWI_MASTER_TRANSMITTER_DATA_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_DATA_ERROR:
	; ERROR								
	ldi temp,(ERROR_TWI_MT_DATA)
	mov FEHLER_CODE, temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																			   
; TWI/I2C START-ROUTINE														   
;																			   
; INPUT:	temp1 <= ADRESSE SLAVE											   
;																			   
; rcall TWI_MASTER_RECEIVER_START	; INP: temp1(SLA) OUT: FEHLER_CODE		   
TWI_MASTER_RECEIVER_START_TEMP1:
	WDR
	; FEHLER lschen					
	clr FEHLER_CODE
	; SRAM lschen						
	STS	(adr_TWI_BEREICH_READ+0),NULL
	STS	(adr_TWI_BEREICH_READ+1),NULL
	STS	(adr_TWI_BEREICH_READ+2),NULL
	STS	(adr_TWI_BEREICH_READ+3),NULL
	STS	(adr_TWI_BEREICH_READ+4),NULL
	STS	(adr_TWI_BEREICH_READ+5),NULL
	STS	(adr_TWI_BEREICH_READ+6),NULL
	STS	(adr_TWI_BEREICH_READ+7),NULL
	STS	(adr_TWI_BEREICH_READ+8),NULL
	STS	(adr_TWI_BEREICH_READ+9),NULL
	STS	(adr_TWI_BEREICH_READ+10),NULL
	STS	(adr_TWI_BEREICH_READ+11),NULL
	STS	(adr_TWI_BEREICH_READ+12),NULL
	STS	(adr_TWI_BEREICH_READ+13),NULL
	STS	(adr_TWI_BEREICH_READ+14),NULL
	; START 											
	ldi temp, (1<<TWINT | 1<<TWEA | 1<<TWSTA | 1<<TWEN )
	STS TWCR, temp	
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	; Check TWSR 										
	LDS temp,TWSR		
	andi temp, 0xF8
	cpi temp,  0x08 								; 0x08 START erfolgreich
	breq TWI_MASTER_RECEIVER_START_w
	cpi temp,  0x10									; 0x10 (repeated START)
	breq TWI_MASTER_RECEIVER_START_w
	rjmp TWI_MASTER_RECEIVER_START_ERROR
TWI_MASTER_RECEIVER_START_w:
	; Load SLA +"R" (R=1)								
	lsl  temp1	; 1 x links
	ori temp1, 0b00000001 
	STS TWDR, temp1
	; senden - Clear TWINT 								
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	STS TWCR, temp
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	; Check TWSR										
	LDS temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x40 	;SLA+R gesendet; ACK empfangen
	brne TWI_MASTER_RECEIVER_START_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_START_ERROR:
	; ERROR						
	ldi temp,(ERROR_TWI_MR_SLA)
	mov FEHLER_CODE, temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; mehrere! Zeichen empfangen werden sollen										
;																				
; OUTPUT: temp1																	
;																				
TWI_MASTER_RECEIVER_DATA_TEMP1:
	; Fehlerprfung										
	tst FEHLER_CODE
	breq TWI_MASTER_RECEIVER_DATA_w
	ret
TWI_MASTER_RECEIVER_DATA_w:
	; senden "ACK" (TWEA=1) fr mehrere Daten-Bytes 	
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	STS TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_wait
	; Check TWSR										
	LDS temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x50	; Data Byte empfangen- ACK zurckgesendet
	breq TWI_MASTER_RECEIVER_DATENEMPFANG_OK
	; ERROR
	ldi temp,(ERROR_TWI_MR_DATA)
	mov FEHLER_CODE, temp	
	clr temp1
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATENEMPFANG_OK:
	; DATEN einlesen							
	LDS temp1, TWDR						
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; nur ein! Zeichen empfangen werden soll, oder wenn								
; das letzte! Zeichen empfangen werden soll										
;																				
; OUTPUT: temp1																	
;																				
TWI_MASTER_RECEIVER_DATA_LAST_BYTE_TEMP1:
	; Fehlerprfung										
	tst FEHLER_CODE
	breq TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_w
	ret
TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_w:
	; senden "NOT ACK" (TWEA=0) fr letztes bzw fr ein Daten-Byte
	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )
	STS TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_wait
	; Check TWSR										
	LDS temp,TWSR
	andi temp, 0xF8
	cpi  temp, 0x58	; letztes Data Byte empfangen-NOT ACK zurckgesendet
	breq TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_OK	; ERROR
	; ERROR
	ldi temp,(ERROR_TWI_MR_DATAEND)
	mov FEHLER_CODE, temp	
	clr temp1
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_OK:
	; DATEN einlesen		
	LDS temp1, TWDR
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; STOP- Routine																   
; 																			   
TWI_STOP:
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)|0<<TWIE	; STOP								
	STS TWCR, temp	
	; kurze Pause
	call wait_1ms
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; ERROR- Routine															   
; 																			   
TWI_ERROR:
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	call wait_100ms	
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; warten, bis das TWINT Flag gesetzt ist									   
; 																			   
TWI_TWINT_wait:
	LDS temp, TWCR
	sbrs temp, TWINT	; nchsten Befehl berspringen, wenn TWINT FLAG  gesetzt ist
	rjmp TWI_TWINT_wait	
ret
; #############################################################################
; #############################################################################
; #############################################################################
