;																		
;																		
; ANALOGUHR - DCF77 - RTC DS1307										
; VIDEOSIGNAL / VOLLGRAFIK / BAS / FBAS /TV-GRAFIC 						
; 																		
; 320x256 PIXEL															
;																		
; ATmega1284P-20MHz														
;																		
; AVR-STUDIO 4															
; Bernhard.Erfurt@gmx.de												
; VERSION: 1 (11/2014)													
;																		

;																		
; Dieses Assembler-Programm	stellt eine Analoguhr auf dem Monitor dar.	
;																		

;																		
; Das TV-Bild generiert ein 64s TIMER (TIMER-0),						
; als Kontrolltimer fr die Taktkorrektur dient der TIMER-2.			
;																		
; Jede der 625 TV-Zeilen wird separat in der "TIM0_COMPA"generiert und 	
; die TV-Norm eingehalten.												
;																		

;																		
; Um wertvolle Takte in der Interruptroutine zu sparen, wird die Zeit 	
; der hinteren Schwarzschulter genutzt, um jede							
; TV-Bild-Daten_Zeile auf Daten zu prfen, liegen keine DATEN vor,		
; dann wird diese Zeile nicht geschrieben (Zeile bleibt Dunkel)			
; Liege nur Daten in einzelnen ZEILEN-Bereichen vor, 					
; wird u.U. nur ein Teil der TV-Zeile geschrieben, das spart Takte !!!	
;																		

;																		
;Nicht alle nutzbaren TV-Zeilen werden abgebildet, ca. 60 TV-ZEILEN		
;bleiben ungenutzt.														
;																		

;																		
; Die Datenausgabe (horizontale Pixel) generiert der PORT-A als 		
; Schieberegister, ist aber	dadurch komplett belegt, 					
; der PORT kann aber bei Bedarf abgendert werden.						
;																		


;																		
; Kurz nach Programmstart synchronisieren sich TIMER-0 und TIMER-1		
; damit der Bildaufbau jitterfrei erfolgen kann.						
;																		

;																		
; GRN leuchtet: alles ok und alles schn								
; GELB leuchtet: Aktivitt -ein Pixel wird gesetzt oder gelscht		
; ROT  leuchtet: 														
; - Programm luft, wenn mglich mit Error weiter						
; - die automatische Taktkorrektur im Interrupt versagt (TIM0_COMPA)	
; - die Zeilenzuordnung stimmt nicht					(TIM0_COMPA)	
; - diverse andere Fehlern												
; - ERROR-CODE lsst sich ggf. anzeigen									
;																		

;																		
; Blinken die LEDs, das ist nicht gut !									
; ROT blinkt: 				Fehler im Programmcheck						
; ROT+GELB blinken: 		WDR-Zeitberschreitung						
; ROT+GELB+GRN blinken: 	unkorrekter Interruptaufruf					
;																		



;												
; Anschlussbelegung PINS						
;												
; PA0 ==> OUT-DATA s/w (0,3V-1V)				
; PA1 ==> 										
; PA2 ==> 										
; PA3 ==> 										
; PA4 ==> 										
; PA5 ==> 										
; PA6 ==> 										
; PA7 ==> 										
;												
; PB0 ==> OUT SYNCHRONIMPULSE (0V-0,3V)			
; PB1 ==> OUT TV-DISPLAY ein (LOW=aus HIGH=EIN)	
; PB2 ==> 										
; PB3 ==> TASTER								
; PB4 ==> 										
; PB5 ==> 										
; PB6 ==> 										
; PB7 ==> 										
;												
; PC0 ==> 										
; PC1 ==> 										
; PC2 ==> 										
; PC3 ==> 										
; PC4 ==>  										
; PC5 ==> LED GRN								
; PC6 ==> LED GELB								
; PC7 ==> LED ROT								
;												
; PD0 ==> 										
; PD1 ==> 										
; PD2 ==> 										
; PD3 ==> 										
; PD4 ==> 										
; PD5 ==>  										
; PD6 ==> 										
; PD7 ==> 										
;												

; ATmega1284P 									
.include "m1284pdef.inc"
; KONSTANTEN									
.equ TAKT							=20000000	; 20 MHz	
; TWI MASTER EINSTELLUNGEN						
.equ TWI_BIT_RATE			=255				; 18	; Bit-Rate							
.equ TWI_PRESCALER			=1					; 1	
; TWI RTC										
.equ TWI_RTC_ADRESSE		=104
; TIMER-WERTE									
.equ TIMER0_CTC_FREQUENZ	=15625/2
.equ TIMER0_N				= 8
.equ TIMER0_CTC				=(TAKT/(2*TIMER0_N*TIMER0_CTC_FREQUENZ))-1
; TIMER-WERTE									
.equ TIMER1_CTC_FREQUENZ	= 50
.equ TIMER1_N				= 64
.equ TIMER1_CTC				=(TAKT/(2*TIMER1_N*TIMER1_CTC_FREQUENZ))-1
; FEHLER-CODES									
.equ ERROR_TWI_MT_START		= 110
.equ ERROR_TWI_MT_SLA		= 112
.equ ERROR_TWI_MT_DATA		= 114
.equ ERROR_TWI_MR_START		= 120
.equ ERROR_TWI_MR_SLA		= 121
.equ ERROR_TWI_MR_DATA		= 122
.equ ERROR_TWI_MR_DATAEND	= 123
; ERROR CODES									
.equ ERROR_CODE_TV_A		=10	; TAKTKORREKTUR				
.equ ERROR_CODE_TV_B		=11	; SPRUNGWEITE				
.equ ERROR_CODE_TV_C		=12	; ZEILENBERWACHUNG			
.equ ERROR_CODE_TV_D		=13	; ZEILENBERWACHUNG			
; DCF-ERROR-CODES								
.equ ERROR_DCF_PHASE						= 10
.equ ERROR_DCF_MINUTENENDE					= 59
.equ ERROR_DCF_NO_ANFANGSBIT				= 100
.equ ERROR_DCF_RESERVEANTENNE				= 215	; nur INFO, kein ERROR	
.equ ERROR_DCF_ZEITUMSTELLUNG				= 216	; nur INFO, kein ERROR	
.equ ERROR_DCF_MESZ							= 118
.equ ERROR_DCF_SCHALTSEKUNDE				= 119
.equ ERROR_DCF_NO_ZEITINFORMATIONSBIT		= 120
.equ ERROR_DCF_MINUTE						= 127
.equ ERROR_DCF_PARITAET_MINUTE				= 128
.equ ERROR_DCF_STUNDE						= 134
.equ ERROR_DCF_PARITAET_STUNDE				= 135
.equ ERROR_DCF_TAG							= 141
.equ ERROR_DCF_WOCHENTAG					= 144
.equ ERROR_DCF_MONAT						= 149
.equ ERROR_DCF_JAHR							= 157
.equ ERROR_DCF_PARITAET_DATUM				= 158
; DCF-EINGANG									
.equ DCF_PIN_NR								= 4
.equ DCF_DDR								= DDRB
.equ DCF_PORT								= PORTB
.equ DCF_PIN								= PINB
; TASTE											
.equ TASTE_PIN_NR							= 2
.equ TASTE_DDR								= DDRB
.equ TASTE_PORT								= PORTB
.equ TASTE_PIN								= PINB
; DISPLAY										
.equ DISPLAY_PIN_NR							= 1
.equ DISPLAY_DDR							= DDRB
.equ DISPLAY_PORT							= PORTB
; DCF-PIN-INVERTIERMODUS						
.equ DCF_PHASENKORREKTUR_WARTEZEIT			= 6	    ; nach einigen Sekunden Phase schieben					
; LED-DCF-GRUEN									
.equ LED_DCF_GRUEN_PIN_NR			= 2
.equ LED_DCF_GRUEN_DDR				= DDRC
.equ LED_DCF_GRUEN_PORT				= PORTC
; LED-DCF-GELB									
.equ LED_DCF_GELB_PIN_NR			= 3
.equ LED_DCF_GELB_DDR				= DDRC
.equ LED_DCF_GELB_PORT				= PORTC
; LED-DCF-ROT									
.equ LED_DCF_ROT_PIN_NR				= 4
.equ LED_DCF_ROT_DDR				= DDRC
.equ LED_DCF_ROT_PORT				= PORTC
; LED-GRUEN										
.equ LED_GRUEN_PIN_NR				= 5
.equ LED_GRUEN_DDR					= DDRC
.equ LED_GRUEN_PORT					= PORTC
; LED-GELB										
.equ LED_GELB_PIN_NR				= 6
.equ LED_GELB_DDR					= DDRC
.equ LED_GELB_PORT					= PORTC
; LED-ROT										
.equ LED_ROT_PIN_NR					= 7
.equ LED_ROT_DDR					= DDRC
.equ LED_ROT_PORT					= PORTC
; TV-DATA										
.equ TV_DATA_PIN_NR					= 0
.equ TV_DATA_DDR					= DDRA
.equ TV_DATA_PORT					= PORTA
; TV-SYNCHRON									
.equ TV_SYNC_PIN_NR					= 0
.equ TV_SYNC_DDR					= DDRB
.equ TV_SYNC_PORT					= PORTB
; TV-BILD										
.equ MITTE_X						=320/2
.equ MITTE_Y						=255/2
; ANALOGUHR										
.equ ANALOGUHR_ZEIGER_LAENGE_ss		=120; ZEIGERLNGE 		
.equ ANALOGUHR_ZEIGER_LAENGE_mm		=110; ZEIGERLNGE 		
.equ ANALOGUHR_ZEIGER_LAENGE_hh		=90 ; ZEIGERLNGE 		
.equ ANALOGUHR_ZEIGER_BREITE_ss		=0	; ZEIGERBREITE 		
.equ ANALOGUHR_ZEIGER_BREITE_mm		=4	; ZEIGERBREITE 		
.equ ANALOGUHR_ZEIGER_BREITE_hh		=4	; ZEIGERBREITE 		
.equ ANALOGUHR_X					=160; POSITION-MITTE	
.equ ANALOGUHR_Y					=128; POSITION-MITTE	
; REGISTER												
.def HILFSREGISTER_R0=R0	; wichtiges Rechenregister	
.def HILFSREGISTER_R1=R1	; wichtiges Rechenregister	
.def HILFSREGISTER_R2=R2	; wichtiges Rechenregister	
.def HILFSREGISTER_R3=R3	; wichtiges Rechenregister	
.def HILFSREGISTER_R4=R4	; wichtiges Rechenregister	
.def HILFSREGISTER_R5=R5	; wichtiges Rechenregister	

.def FEHLER_CODE	= R5

.def SREG_KOPIE		= R6	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def temp_KOPIE		= R7	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def temp1_KOPIE	= R8	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def temp2_KOPIE	= R9	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def ZL_KOPIE		= R10	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def ZH_KOPIE		= R11	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def TV_ZEILE_NR_L 	= R12	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def TV_ZEILE_NR_H 	= R13	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !

.def NULL 			= R14	; ist immer NULL 	
.def EINS 			= R15	; ist immer EINS 	

.def temp 			= R16
.def temp1 			= R17
.def temp2 			= R18
.def temp3 			= R19
.def temp4 			= R20
.def temp5 			= R21
.def temp6 			= R22
.def temp7 			= R23
.def temp8 			= R24
.def temp9 			= R25

; SRAM-BELEGUNG Adr:0x0100-0x40FF(256-16.639=16383)		
.equ adr_DATEN						=SRAM_START		

.equ adr_FREIGABE_PROGRAMM			=SRAM_START+1		; diverse Zeitsteuerungen	

.equ adr_TASTE						=SRAM_START+3

.equ adr_ERROR_CODE_TV				=SRAM_START+10		; ERROR CODE TV				
.equ adr_ERROR_WERT_TV				=SRAM_START+11		; ERROR CODE TV (WERT)		

.equ adr_STOP_UHR_ss10				=SRAM_START+14		; STOP-UHR					
.equ adr_STOP_UHR_ERGEBNIS			=SRAM_START+15		; STOP-UHR					

.equ adr_ss100						=SRAM_START+20		; Sekunden 10-tel 			
.equ adr_ss							=SRAM_START+21		; Sekunden					
.equ adr_mm							=SRAM_START+22		; Minuten					
.equ adr_hh							=SRAM_START+23		; Stunden					
.equ adr_TAG						=SRAM_START+24		; TAG						
.equ adr_MONAT						=SRAM_START+25		; Monat						
.equ adr_JAHR						=SRAM_START+26		; JAHR						
.equ adr_WOCHENTAG					=SRAM_START+27		; 1=Mo 7=So					
.equ adr_MESZ						=SRAM_START+28

.equ adr_GRAFIC_FARBE				=SRAM_START+100		; 0=wei 1=schwarz			
.equ adr_GRAFIC_PUNKT_GROESSE		=SRAM_START+101		; PUNKT-GRSSE 0...9		
.equ adr_GRAFIC_KREIS_RADIUS		=SRAM_START+102		; RADIUS					
.equ adr_GRAFIC_WINKEL_L			=SRAM_START+103		; WINKEL					
.equ adr_GRAFIC_WINKEL_H			=SRAM_START+104		; WINKEL					
.equ adr_GRAFIC_LAENGE_L			=SRAM_START+105		; STRECKEN-LNGE			
.equ adr_GRAFIC_LAENGE_H			=SRAM_START+106		; STRECKEN-LNGE			

.equ adr_GRAFIC_ASCII_GROESSE		=SRAM_START+110		;
.equ adr_GRAFIC_ASCII_INVERS		=SRAM_START+111		;
.equ adr_GRAFIC_ASCII_CURSOR_Y_L	=SRAM_START+112		; CURSOR TV-ZEILE vom CURSOR	
.equ adr_GRAFIC_ASCII_CURSOR_Y_H	=SRAM_START+113	
.equ adr_GRAFIC_ASCII_CURSOR_X		=SRAM_START+114		; CURSOR-SPALTE (0..19)(0..39)	


.equ adr_GRAFIC_X1_L				=SRAM_START+120		; X1-KOORDINATEN
.equ adr_GRAFIC_X1_H				=SRAM_START+121
.equ adr_GRAFIC_Y1_L				=SRAM_START+122		; Y1-KOORDINATEN
.equ adr_GRAFIC_Y1_H				=SRAM_START+123
.equ adr_GRAFIC_X2_L				=SRAM_START+124		; X2-KOORDINATEN
.equ adr_GRAFIC_X2_H				=SRAM_START+125
.equ adr_GRAFIC_Y2_L				=SRAM_START+126		; Y2-KOORDINATEN
.equ adr_GRAFIC_Y2_H				=SRAM_START+127
.equ adr_GRAFIC_X3_L				=SRAM_START+128		; X3-KOORDINATEN
.equ adr_GRAFIC_X3_H				=SRAM_START+129
.equ adr_GRAFIC_Y3_L				=SRAM_START+130		; Y3-KOORDINATEN
.equ adr_GRAFIC_Y3_H				=SRAM_START+131
.equ adr_GRAFIC_X4_L				=SRAM_START+132		; X4-KOORDINATEN
.equ adr_GRAFIC_X4_H				=SRAM_START+133
.equ adr_GRAFIC_Y4_L				=SRAM_START+134		; Y4-KOORDINATEN
.equ adr_GRAFIC_Y4_H				=SRAM_START+135

.equ adr_GRAFIC_X1_LINIE_L			=SRAM_START+150		; fr LINIEN-BERCHNUNG		
.equ adr_GRAFIC_X1_LINIE_H			=SRAM_START+151
.equ adr_GRAFIC_Y1_LINIE_L			=SRAM_START+152
.equ adr_GRAFIC_Y1_LINIE_H			=SRAM_START+153
.equ adr_GRAFIC_X2_LINIE_L			=SRAM_START+154
.equ adr_GRAFIC_X2_LINIE_H			=SRAM_START+155
.equ adr_GRAFIC_Y2_LINIE_L			=SRAM_START+156
.equ adr_GRAFIC_Y2_LINIE_H			=SRAM_START+157

.equ adr_GRAFIC_LINIE_DELTA_X_L		=SRAM_START+160		; fr LINIEN-BERCHNUNG		
.equ adr_GRAFIC_LINIE_DELTA_X_H		=SRAM_START+161
.equ adr_GRAFIC_LINIE_DELTA_Y_L		=SRAM_START+162
.equ adr_GRAFIC_LINIE_DELTA_Y_H		=SRAM_START+163

.equ adr_GRAFIC_X1_KOPIE_L			=SRAM_START+250		; X1-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_X1_KOPIE_H			=SRAM_START+251
.equ adr_GRAFIC_Y1_KOPIE_L			=SRAM_START+252		; Y1-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_Y1_KOPIE_H			=SRAM_START+253
.equ adr_GRAFIC_X2_KOPIE_L			=SRAM_START+254		; X2-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_X2_KOPIE_H			=SRAM_START+255
.equ adr_GRAFIC_Y2_KOPIE_L			=SRAM_START+256		; Y2-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_Y2_KOPIE_H			=SRAM_START+257
.equ adr_GRAFIC_X3_KOPIE_L			=SRAM_START+258		; X3-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_X3_KOPIE_H			=SRAM_START+259
.equ adr_GRAFIC_Y3_KOPIE_L			=SRAM_START+260		; Y3-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_Y3_KOPIE_H			=SRAM_START+261
.equ adr_GRAFIC_X4_KOPIE_L			=SRAM_START+262		; X4-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_X4_KOPIE_H			=SRAM_START+263
.equ adr_GRAFIC_Y4_KOPIE_L			=SRAM_START+264		; Y4-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_Y4_KOPIE_H			=SRAM_START+265

.equ adr_ANALOGUHR_OLD_ss			=SRAM_START+300		; ANALOGUHR					
.equ adr_ANALOGUHR_OLD_mm			=SRAM_START+301		; ANALOGUHR					
.equ adr_ANALOGUHR_OLD_hh			=SRAM_START+302		; ANALOGUHR					

.equ adr_RTC_WRITE_AKTIV			=SRAM_START+350		; RTC						

.equ adr_TWI_BEREICH_READ			=SRAM_START+400		; BEREICH fr Master TWI/I2C	

.equ adr_DCF_ss100					=SRAM_START+500			; 100-tel Sekunden				
.equ adr_DCF_ss						=SRAM_START+501			; SEKUNDEN						
.equ adr_DCF_mm						=SRAM_START+502			; Minuten 	aus DCF-Protokoll	
.equ adr_DCF_hh						=SRAM_START+503			; Stunden 	aus DCF-Protokoll	
.equ adr_DCF_TAG					=SRAM_START+504			; Tag	 	aus DCF-Protokoll	
.equ adr_DCF_WOCHENTAG				=SRAM_START+505			; WochenTag aus DCF-Protokoll	
.equ adr_DCF_MONAT					=SRAM_START+506			; Monat	 	aus DCF-Protokoll	
.equ adr_DCF_JAHR					=SRAM_START+507			; Jahr	 	 aus DCF-Protokoll	
.equ adr_DCF_MESZ					=SRAM_START+508			; SOMMERZEIT aus DCF-Protokoll	

.equ adr_DCF_PHASE					=SRAM_START+510			; 
.equ adr_DCF_PHASE_WAIT				=SRAM_START+512			; 
.equ adr_DCF_SIGNAL_INVERTIEREN		=SRAM_START+514			; 
.equ adr_DCF_SIGNAL_ZAEHLER			=SRAM_START+516			; 
.equ adr_DCF_TRAEGER				=SRAM_START+518			; 
.equ adr_DCF_SAMPLES_A				=SRAM_START+520			; SAMPLES_A (100ms)				
.equ adr_DCF_SAMPLES_B				=SRAM_START+522			; SAMPLES_B (200ms)				
.equ adr_DCF_MINUTENENDE			=SRAM_START+524
.equ adr_DCF_PROTOKOLL_OK			=SRAM_START+526
.equ adr_DCF_NO_BERECHNUNG			=SRAM_START+528
.equ adr_DCF_ERROR_CODE				=SRAM_START+530			; DCF-ERROR-CODE				
.equ adr_DCF_GUELTIG				=SRAM_START+532			; DCF-Protokoll ist geprft und gltig			
.equ adr_DCF_HEUTE_EMPFANGEN		=SRAM_START+534			; DCF-Protokoll heute schon komplett empfangen	

.equ adr_DCF_BEREICH60				=SRAM_START+536	 		; 60 BYTES fr DCF-FELD	

.equ adr_TV_ZEIGER_DATA_L			=SRAM_START+810			; ZEIGER fr BILD-DATEN			
.equ adr_TV_ZEIGER_DATA_H			=SRAM_START+811

.equ adr_TV_BILDBEREICH				=SRAM_START+1000 		;(40*256=10240 Bytes)			
.equ adr_TV_BILDBEREICH_ENDE		=SRAM_START+1000-1+40*256

.equ adr_SRAM_CHECK					=SRAM_START+15500		; SRAM CHECK muss immer "S" sein	




.cseg						;Beginn eines Code-Segmentes
.org 0						;Startadresse=0				
.org 0x0000 rjmp RESET 			 ; Reset									
.org 0x0002 rjmp INTERRUPT_ERROR ;rjmp INT0 ; IRQ0							
.org 0x0004 rjmp INTERRUPT_ERROR ;rjmp INT1 ; IRQ1							
.org 0x0006 rjmp INTERRUPT_ERROR ;rjmp INT2 ; IRQ2							
.org 0x0008 rjmp INTERRUPT_ERROR ;rjmp PCINT0 ; PCINT0						
.org 0x000A rjmp INTERRUPT_ERROR ;rjmp PCINT1 ; PCINT1						
.org 0x000C rjmp INTERRUPT_ERROR ;rjmp PCINT2 ; PCINT2						
.org 0x000E rjmp INTERRUPT_ERROR ;rjmp PCINT3 ; PCINT3						
.org 0x0010 jmp WDT 		; Watchdog Timeout								
.org 0x0012 rjmp INTERRUPT_ERROR ;rjmp TIM2_COMPA ; Timer2 CompareA			
.org 0x0014 rjmp INTERRUPT_ERROR ;rjmp TIM2_COMPB ; Timer2 CompareB			
.org 0x0016 rjmp INTERRUPT_ERROR ;rjmp TIM2_OVF 	; Timer2 Overflow		
.org 0x0018 rjmp INTERRUPT_ERROR ;rjmp TIM1_CAPT ; Timer1 Capture			
.org 0x001A rjmp TIM1_COMPA ; Timer1 CompareA			
.org 0x001C rjmp INTERRUPT_ERROR ;rjmp TIM1_COMPB ; Timer1 CompareB			
.org 0x001E rjmp INTERRUPT_ERROR ;TIM1_OVF 	; Timer1 Overflow				
.org 0x0020 rjmp TIM0_COMPA ; Timer0 CompareA								
.org 0x0022 rjmp INTERRUPT_ERROR ;rjmp TIM0_COMPB ; Timer0 CompareB			
.org 0x0024 rjmp INTERRUPT_ERROR ;rjmp TIM0_OVF 	; Timer0 Overflow		
.org 0x0026 rjmp INTERRUPT_ERROR ;rjmp SPI_STC ; SPI Transfer Complete		
.org 0x0028 rjmp INTERRUPT_ERROR ;rjmp USART0_RXC ; USART0 RX Complete		
.org 0x002A rjmp INTERRUPT_ERROR ;rjmp USART0_UDRE ; USART0,UDR Empty		
.org 0x002C rjmp INTERRUPT_ERROR ;rjmp USART0_TXC ; USART0 TX Complete		
.org 0x002E rjmp INTERRUPT_ERROR ;rjmp ANA_COMP ; Analog Comparator			
.org 0x0030 rjmp INTERRUPT_ERROR ;rjmp ADC ; ADC Conversion Complete		
.org 0x0032 rjmp INTERRUPT_ERROR ;rjmp EE_RDY ; EEPROM Ready				
.org 0x0034 rjmp INTERRUPT_ERROR ;rjmp TWI ; 2-wire Serial					
.org 0x0036 rjmp INTERRUPT_ERROR ;rjmp SPM_RDY ; SPM Ready					
.org 0x0038 rjmp INTERRUPT_ERROR ;rjmp USART1_RXC ; USART1 RX Complete		
.org 0x003A rjmp INTERRUPT_ERROR ;rjmp USART1_UDRE ; USART1,UDR Empty		
.org 0x003C rjmp INTERRUPT_ERROR ;rjmp USART1_TXC ; USART1 TX Complete		
.org 0x003E rjmp INTERRUPT_ERROR ;rjmp TIM3_CAPT ; Timer3 Capture			
.org 0x0040 rjmp INTERRUPT_ERROR ;rjmp TIM3_COMPA ; Timer3 CompareA			
.org 0x0042 rjmp INTERRUPT_ERROR ;rjmp TIM3_COMPB ; Timer3 CompareB			
.org 0x0044 rjmp INTERRUPT_ERROR ;rjmp TIM3_OVF 	; Timer3 Overflow		
; Unkorrekten Interrupt Aufruf 						
INTERRUPT_ERROR:
	CLI											; INTERRUPTS sperren	
	cbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)		; GRN an   
	cbi (LED_GELB_PORT),(LED_GELB_PIN_NR)		; GELB an	
	cbi (LED_ROT_PORT),(LED_ROT_PIN_NR)			; ROT  an	
	call wait_100ms
	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)		; GRN aus	
	sbi (LED_GELB_PORT),(LED_GELB_PIN_NR)		; GELB aus  
	sbi (LED_ROT_PORT),(LED_ROT_PIN_NR)			; ROT  aus	
	call wait_100ms
	rjmp INTERRUPT_ERROR
;-------------------------------------------------------------------------------
RESET:                        
; STACK initialisieren								
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; SRAM_CLEAR										
	call SRAM_CLEAR
; STARTWERTE										
	; NULL				
	clr NULL
	; EINS				
	ldi temp,1
	mov EINS,temp
	; SRAM CHECK		
	ldi temp,'S'
	STS(adr_SRAM_CHECK),temp
	; TV_ZEILE			
	ldi temp1,LOW (-1)
	ldi temp2,HIGH(-1)
	mov TV_ZEILE_NR_L,temp1
	mov TV_ZEILE_NR_H,temp2
	; DATUM				
	ldi temp,1
	STS(adr_TAG),temp
	ldi temp,1
	STS(adr_MONAT),temp
	ldi temp,14
	STS(adr_JAHR),temp
	ldi temp,6
	STS(adr_WOCHENTAG),temp
	; ZEIT				
	ldi temp,0
	STS(adr_hh),temp	; hh	
	ldi temp,0
	STS(adr_mm),temp	; mm	
	ldi temp,0
	STS(adr_ss),temp	; ss	
	ldi temp,0
	STS(adr_MESZ),temp	; MESZ	
	; ANALOGUHR			
	ldi temp,99
	STS(adr_ANALOGUHR_OLD_ss),temp
	STS(adr_ANALOGUHR_OLD_mm),temp
	STS(adr_ANALOGUHR_OLD_hh),temp
	; DCF				
	ldi temp,(DCF_PHASENKORREKTUR_WARTEZEIT)
	STS(adr_DCF_PHASE_WAIT),temp
; PORT A+B+C+D initialisieren (alles EINGNGE+PULL-UP)		
	ldi temp,255
	out DDRA, NULL
	out PORTA,temp
	out DDRB, NULL
	out PORTB,temp
	out DDRC, NULL
	out PORTC,temp
	out DDRD, NULL
	out PORTD,temp
; LED_INITIALISIERUNG										
	call LED_INITIALISIERUNG
; TV_PIN_INITIALISIERUNG									
	call TV_PIN_INITIALISIERUNG
; WDR_INITIALISIERUNG										
	call WDR_INITIALISIERUNG
; NEUSTART_MIT_WDR											
	call NEUSTART_MIT_WDR
; LED_TEST													
	call LED_TEST
; PROGRAMM_CHECK											
	call PROGRAMM_CHECK
; TWI_INI													
	call TWI_INI
; DCF_INITIALISIERUNG										
	call DCF_INITIALISIERUNG
; TIMER INITALISIERUNG										
	call TIMER0_INITIALISIERUNG_CTC
	call TIMER2_INITIALISIERUNG
	call TIMER1_INITIALISIERUNG_CTC
; Interrupts freigeben										
	sei
; ##############################################################################
; ##############################################################################
; ##############################################################################
; EINSCHWINGEN												
	call GRAFIC_CLEAR
	call TEST_BITMUSTER
	call PROGRAMM_CHECK
	call wait_1s
; DISPLAY_ON und warten...									
	call DISPLAY_ON
	call wait_1s
	call wait_1s
	call wait_1s
	call wait_1s
	call wait_1s
	call wait_1s
	call wait_1s
	call wait_1s
	call wait_1s
	call wait_1s

	sbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  aus		
	STS(adr_ERROR_CODE_TV),NULL					; ERROR CLEAR	
	STS(adr_ERROR_WERT_TV),NULL					; ERROR CLEAR	
; RTC auslesen													
	call DS1307_UHR_AUSLESEN
;	call DS1307_UHR_STELLEN_INITIALISIEREN
; SYSTEM													
	call GRAFIC_CLEAR
	call ANZEIGE_SYSTEM
	call wait_1s
; alle FIGUREN												
	call GRAFIC_CLEAR
	call TEST_ALLE_FIGUREN
	call wait_1s
; BEGRSSUNG												
	call GRAFIC_CLEAR
	call ANZEIGE_BEGRUESSUNG
	call wait_1s
; TEST-GRAFIC												
	call wait_100ms
	call GRAFIC_CLEAR
; BEREIT													
	sbi (LED_DCF_GRUEN_PORT),(LED_DCF_GRUEN_PIN_NR)	; DCF-GRUEN	
	cbi (LED_DCF_ROT_PORT),(LED_DCF_ROT_PIN_NR)	 	; DCF-ROT	
	cbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)			; GRN an   
;	call STOP_PROGRAMM
; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	WDR										; Watch-Dog-RESET	
	call ABFRAGE_TASTE						; TASTE abfragen	
	LDS temp,(adr_FREIGABE_PROGRAMM)
	tst temp
	breq LOOP
	STS(adr_FREIGABE_PROGRAMM),NULL
;-------------------------------------------------------------------------------
	call ANALOGUHR
	call DS1307_RTC_STELLEN
	call DISPLAY_AUTOMATIK_OFF_ON	; 00...07 UHR off
	call DISPLAY_AUTOMATIK_ON
	call PROGRAMM_CHECK
	call ANZEIGE_ERROR_CODE_TV
;	call ANZEIGE_HAUPTMENUE
;	call TEST_STERNEN_HIMMEL
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "TV_TIMER.asm"					; TIMER+UHR						
.include "TV_WAIT_20MHZ.asm"			; 20 MHz						
.include "TV_TV_MACRO.asm"				; MAKROS						
.include "TV_TV_BILD_TIMER.asm"			; TIMER+BILDAUFBAU				
.include "TV_TWI.asm"					; TWI							
.include "TV_TWI_DS1307.asm"			; TWI RTC DS1307				
.include "TV_DCF77.asm"					; DCF77							
.include "TV_GRAFIC_PUNKT.asm"			; PUNKT							
.include "TV_GRAFIC_LINIE.asm" 			; LINIE							
.include "TV_GRAFIC_KREIS.asm" 			; KREIS							
.include "TV_ASCII.asm"					; ASCII							
.include "TV_ANALOGUHR.asm"				; ANALOGUHR						
.include "TV_GRAFIC_TESTS.asm" 			; TESTBILDER					
.include "TV_ANZEIGE.asm"				; TEXTE							
.include "TV_ANZEIGE_KURZ.asm"			; TEXTE							
.include "TV_ANZEIGE_ZAHLEN.asm"		; ZAHLEN						
.include "TV_SONSTIGES.asm"				; sonstiges						
.include "TV_FUNKTIONEN.asm"			; FUNKTIONEN					
.include "TV_TABELLE_ALPHABET.asm"		; TABELLE Alphabet				
.include "TV_TABELLE_ASCII_8x12.asm"	; TABELLE (umfangreich)			
.include "TV_TABELLE_ASCII_16x24.asm"	; TABELLE (umfangreich)			
.include "TV_TABELLE_SINUS.asm"			; TABELLE (umfangreich)			
;-------------------------------------------------------------------------------
TEXT_BEGRUESSUNG_1:
.db "  WEIHNACHTEN 2014  ",0
TEXT_BEGRUESSUNG_2:
.db " ",0
TEXT_BEGRUESSUNG_3:
.db "    ANALOGUHR",0
TEXT_BEGRUESSUNG_4:
.db " ",0
TEXT_BEGRUESSUNG_5:
.db "  ATmega1284P 20MHz",0
TEXT_BEGRUESSUNG_6:
.db " ",0
TEXT_BEGRUESSUNG_7:
.db "  DCF-77 RTC DS1307",0
TEXT_BEGRUESSUNG_8:
.db " ",0
TEXT_BEGRUESSUNG_9:
.db " ",0
TEXT_BEGRUESSUNG_10:
.db "   V2 12/2014",0
;-------------------------------------------------------------------------------


